@@ -35,7 +35,6 @@ use Bugzilla::Config;
 use Bugzilla::Constants;
 use Bugzilla::Error;
 use Bugzilla::Util;
-use Bugzilla::Token;
 
 sub login {
     my ($class, $type) = @_;
@@ -71,18 +70,16 @@ sub login {
         # subsequent login
         trick_taint($ipaddr);
 
-        my $logincookie = Bugzilla::Token::GenerateUniqueToken('logincookies', 'cookie');
-
-        $dbh->do("INSERT INTO logincookies (cookie, userid, ipaddr, lastused)
-                 VALUES (?, ?, ?, NOW())",
+        $dbh->do("INSERT INTO logincookies (userid, ipaddr, lastused)
+                 VALUES (?, ?, NOW())",
                  undef,
-                 $logincookie, $userid, $ipaddr);
+                 $userid, $ipaddr);
+        my $logincookie = $dbh->bz_last_key('logincookies', 'cookie');
 
         # Remember cookie only if admin has told so
         # or admin didn't forbid it and user told to remember.
         if ((Param('rememberlogin') eq 'on') ||
             ((Param('rememberlogin') ne 'off') &&
-             $cgi->param('Bugzilla_remember') &&
              ($cgi->param('Bugzilla_remember') eq 'on'))) {
             $cgi->send_cookie(-name => 'Bugzilla_login',
                               -value => $userid,
@@ -198,7 +195,6 @@ sub can_logout { return 1; }
 sub logout {
     my ($class, $user, $option) = @_;
     my $dbh = Bugzilla->dbh;
-    my $cgi = Bugzilla->cgi;
     $option = LOGOUT_ALL unless defined $option;
 
     if ($option == LOGOUT_ALL) {
@@ -207,18 +203,9 @@ sub logout {
             return;
     }
 
-    # The LOGOUT_*_CURRENT options require the current login cookie.
-    # If a new cookie has been issued during this run, that's the current one.
-    # If not, it's the one we've received.
-    my $cookie;
-    foreach (@{$cgi->{'Bugzilla_cookie_list'}}) {
-        if ($_->name() eq 'Bugzilla_logincookie') {
-            $cookie = $_->value();
-            last;
-        }
-    }
-    $cookie ||= $cgi->cookie("Bugzilla_logincookie");
-    trick_taint($cookie);
+    # The LOGOUT_*_CURRENT options require a cookie 
+    my $cookie = Bugzilla->cgi->cookie("Bugzilla_logincookie");
+    detaint_natural($cookie);
 
     # These queries use both the cookie ID and the user ID as keys. Even
     # though we know the userid must match, we still check it in the SQL
