@@ -23,7 +23,6 @@
 #                 Dave Lawrence <dkl@redhat.com>
 #                 Tomas Kopal <Tomas.Kopal@altap.cz>
 #                 Max Kanat-Alexander <mkanat@bugzilla.org>
-#                 Lance Larsh <lance.larsh@oracle.com>
 
 =head1 NAME
 
@@ -43,13 +42,12 @@ package Bugzilla::DB::Mysql;
 
 use strict;
 
-use Bugzilla::Util;
 use Bugzilla::Error;
 
 # This module extends the DB interface via inheritance
 use base qw(Bugzilla::DB);
 
-use constant REQUIRED_VERSION => '4.0.14';
+use constant REQUIRED_VERSION => '3.23.41';
 use constant PROGRAM_NAME => 'MySQL';
 use constant MODULE_NAME  => 'Mysql';
 use constant DBD_VERSION  => '2.9003';
@@ -84,15 +82,11 @@ sub bz_last_key {
 }
 
 sub sql_regexp {
-    my ($self, $expr, $pattern) = @_;
-
-    return "$expr REGEXP $pattern";
+    return "REGEXP";
 }
 
 sub sql_not_regexp {
-    my ($self, $expr, $pattern) = @_;
-
-    return "$expr NOT REGEXP $pattern";
+    return "NOT REGEXP";
 }
 
 sub sql_limit {
@@ -114,17 +108,7 @@ sub sql_string_concat {
 sub sql_fulltext_search {
     my ($self, $column, $text) = @_;
 
-    # Add the boolean mode modifier if the search string contains
-    # boolean operators.
-    my $mode = ($text =~ /[+-<>()~*"]/ ? "IN BOOLEAN MODE" : "");
-
-    # quote the text for use in the MATCH AGAINST expression
-    $text = $self->quote($text);
-
-    # untaint the text, since it's safe to use now that we've quoted it
-    trick_taint($text);
-
-    return "MATCH($column) AGAINST($text $mode)";
+    return "MATCH($column) AGAINST($text)";
 }
 
 sub sql_istring {
@@ -133,12 +117,6 @@ sub sql_istring {
     return $string;
 }
 
-sub sql_from_days {
-    my ($self, $days) = @_;
-
-    return "FROM_DAYS($days)";
-}
-
 sub sql_to_days {
     my ($self, $date) = @_;
 
@@ -154,9 +132,9 @@ sub sql_date_format {
 }
 
 sub sql_interval {
-    my ($self, $interval, $units) = @_;
+    my ($self, $interval) = @_;
     
-    return "INTERVAL $interval $units";
+    return "INTERVAL $interval";
 }
 
 sub sql_position {
@@ -214,7 +192,7 @@ sub bz_unlock_tables {
     }
 }
 
-# As Bugzilla currently runs on MyISAM storage, which does not support
+# As Bugzilla currently runs on MyISAM storage, which does not supprt
 # transactions, these functions die when called.
 # Maybe we should just ignore these calls for now, but as we are not
 # using transactions in MySQL yet, this just hints the developers.
@@ -268,17 +246,6 @@ sub bz_setup_database {
         print "\nISAM->MyISAM table conversion done.\n\n";
     }
 
-    # There is a bug in MySQL 4.1.0 - 4.1.15 that makes certain SELECT
-    # statements fail after a SHOW TABLE STATUS: 
-    # http://bugs.mysql.com/bug.php?id=13535
-    # This is a workaround, a dummy SELECT to reset the LAST_INSERT_ID.
-    my @tables = $self->bz_table_list_real();
-    if (grep($_ eq 'bugs', @tables)
-        && $self->bz_column_info_real("bugs", "bug_id"))
-    {
-        $self->do('SELECT 1 FROM bugs WHERE bug_id IS NULL');
-    }
-
     # Versions of Bugzilla before the existence of Bugzilla::DB::Schema did 
     # not provide explicit names for the table indexes. This means
     # that our upgrades will not be reliable, because we look for the name
@@ -294,31 +261,11 @@ sub bz_setup_database {
     # has existed at least since Bugzilla 2.8, and probably earlier.
     # For fixing the inconsistent naming of Schema indexes,
     # we also check for one of those inconsistently-named indexes.
-    if (grep($_ eq 'bugs', @tables)
-        && ($self->bz_index_info_real('bugs', 'assigned_to')
-            || $self->bz_index_info_real('flags', 'flags_bidattid_idx')) )
+    my @tables = $self->bz_table_list_real();
+    if ( scalar(@tables) && 
+         ($self->bz_index_info_real('bugs', 'assigned_to') ||
+          $self->bz_index_info_real('flags', 'flags_bidattid_idx')) )
     {
-
-        # This is a check unrelated to the indexes, to see if people are
-        # upgrading from 2.18 or below, but somehow have a bz_schema table
-        # already. This only happens if they have done a mysqldump into
-        # a database without doing a DROP DATABASE first.
-        # We just do the check here since this check is a reliable way
-        # of telling that we are upgrading from a version pre-2.20.
-        if (grep($_ eq 'bz_schema', $self->bz_table_list_real())) {
-            die("\nYou are upgrading from a version before 2.20, but the"
-              . " bz_schema\ntable already exists. This means that you"
-              . " restored a mysqldump into\nthe Bugzilla database without"
-              . " first dropping the already-existing\nBugzilla database,"
-              . " at some point. Whenever you restore a Bugzilla\ndatabase"
-              . " backup, you must always drop the entire database first.\n\n"
-              . "Please drop your Bugzilla database and restore it from a"
-              . " backup that\ndoes not contain the bz_schema table. If for"
-              . " some reason you cannot\ndo this, you can connect to your"
-              . " MySQL database and drop the bz_schema\ntable, as a last"
-              . " resort.\n");
-        }
-
         my $bug_count = $self->selectrow_array("SELECT COUNT(*) FROM bugs");
         # We estimate one minute for each 3000 bugs, plus 3 minutes just
         # to handle basic MySQL stuff.
@@ -527,51 +474,9 @@ sub bz_setup_database {
                                {TYPE => 'DATETIME', NOTNULL => 1});
     }
 
-    # 2005-09-24 - bugreport@peshkin.net, bug 307602
-    # Make sure that default 4G table limit is overridden
-    my $row = $self->selectrow_hashref("SHOW TABLE STATUS LIKE 'attach_data'");
-    if ($$row{'Create_options'} !~ /MAX_ROWS/i) {
-        print "Converting attach_data maximum size to 100G...\n";
-        $self->do("ALTER TABLE attach_data
-                   AVG_ROW_LENGTH=1000000,
-                   MAX_ROWS=100000");
-    }
-
 }
 
 
-sub bz_enum_initial_values {
-    my ($self, $enum_defaults) = @_;
-    my %enum_values = %$enum_defaults;
-    # Get a complete description of the 'bugs' table; with DBD::MySQL
-    # there isn't a column-by-column way of doing this.  Could use
-    # $dbh->column_info, but it would go slower and we would have to
-    # use the undocumented mysql_type_name accessor to get the type
-    # of each row.
-    my $sth = $self->prepare("DESCRIBE bugs");
-    $sth->execute();
-    # Look for the particular columns we are interested in.
-    while (my ($thiscol, $thistype) = $sth->fetchrow_array()) {
-        if (defined $enum_values{$thiscol}) {
-            # this is a column of interest.
-            my @value_list;
-            if ($thistype and ($thistype =~ /^enum\(/)) {
-                # it has an enum type; get the set of values.
-                while ($thistype =~ /'([^']*)'(.*)/) {
-                    push(@value_list, $1);
-                    $thistype = $2;
-                }
-            }
-            if (@value_list) {
-                # record the enum values found.
-                $enum_values{$thiscol} = \@value_list;
-            }
-        }
-    }
-
-    return \%enum_values;
-}
-
 #####################################################################
 # MySQL-specific Database-Reading Methods
 #####################################################################
