@@ -35,7 +35,8 @@ See below for more information.
 =item *
 
 Prior to calling routines in this module, it's assumed that you have
-already done a C<require globals.pl>.
+already done a C<require CGI.pl>.  This will eventually change in a
+future version when CGI.pl is removed.
 
 =item *
 
@@ -320,9 +321,6 @@ to extract flag type IDs from form field names by matching columns
 whose name looks like "flag_type-nnn", where "nnn" is the ID,
 and returning just the ID portion of matching field names.
 
-If the attachment is new, it has no ID yet and $attach_id is set
-to -1 to force its check anyway.
-
 =back
 
 =cut
@@ -344,9 +342,17 @@ sub validate {
     # this bug/attachment. This check will be done later when
     # processing new flags, see Flag::FormToNewFlags().
 
+    # All flag types have to be active
+    my $inactive_flagtypes =
+        $dbh->selectrow_array("SELECT 1 FROM flagtypes
+                               WHERE id IN (" . join(',', @ids) . ")
+                               AND is_active = 0 " .
+                               $dbh->sql_limit(1));
+
+    ThrowCodeError("flag_type_inactive") if $inactive_flagtypes;
+
     foreach my $id (@ids) {
         my $status = $cgi->param("flag_type-$id");
-        my @requestees = $cgi->param("requestee_type-$id");
         
         # Don't bother validating types the user didn't touch.
         next if $status eq "X";
@@ -356,10 +362,6 @@ sub validate {
         $flag_type 
           || ThrowCodeError("flag_type_nonexistent", { id => $id });
 
-        # Make sure the flag type is active.
-        $flag_type->{'is_active'}
-          || ThrowCodeError('flag_type_inactive', {'type' => $flag_type->{'name'}});
-
         # Make sure the value of the field is a valid status.
         grep($status eq $_, qw(X + - ?))
           || ThrowCodeError("flag_status_invalid", 
@@ -373,53 +375,48 @@ sub validate {
         
         # Make sure the user didn't specify a requestee unless the flag
         # is specifically requestable.
+        my $new_requestee = trim($cgi->param("requestee_type-$id") || '');
+
         if ($status eq '?'
             && !$flag_type->{is_requesteeble}
-            && scalar(@requestees) > 0)
+            && $new_requestee)
         {
-            ThrowCodeError("flag_requestee_disabled", { type => $flag_type });
+            ThrowCodeError("flag_requestee_disabled",
+                           { name => $flag_type->{name} });
         }
 
-        # Make sure the user didn't enter multiple requestees for a flag
-        # that can't be requested from more than one person at a time.
-        if ($status eq '?'
-            && !$flag_type->{is_multiplicable}
-            && scalar(@requestees) > 1)
-        {
-            ThrowUserError("flag_not_multiplicable", { type => $flag_type });
-        }
-
-        # Make sure the requestees are authorized to access the bug
+        # Make sure the requestee is authorized to access the bug
         # (and attachment, if this installation is using the "insider group"
         # feature and the attachment is marked private).
-        if ($status eq '?' && $flag_type->{is_requesteeble}) {
-            foreach my $login (@requestees) {
-                # We know the requestee exists because we ran
-                # Bugzilla::User::match_field before getting here.
-                my $requestee = Bugzilla::User->new_from_login($login);
-    
-                # Throw an error if the user can't see the bug.
-                if (!$requestee->can_see_bug($bug_id)) {
-                    ThrowUserError("flag_requestee_unauthorized",
-                                   { flag_type => $flag_type,
-                                     requestee => $requestee,
-                                     bug_id    => $bug_id,
-                                     attach_id => $attach_id });
-                }
-                
-                # Throw an error if the target is a private attachment and
-                # the requestee isn't in the group of insiders who can see it.
-                if ($attach_id
-                    && Param("insidergroup")
-                    && $cgi->param('isprivate')
-                    && !$requestee->in_group(Param("insidergroup")))
-                {
-                    ThrowUserError("flag_requestee_unauthorized_attachment",
-                                   { flag_type => $flag_type,
-                                     requestee => $requestee,
-                                     bug_id    => $bug_id,
-                                     attach_id => $attach_id });
-                }
+        if ($status eq '?'
+            && $flag_type->{is_requesteeble}
+            && $new_requestee)
+        {
+            # We know the requestee exists because we ran
+            # Bugzilla::User::match_field before getting here.
+            my $requestee = Bugzilla::User->new_from_login($new_requestee);
+
+            # Throw an error if the user can't see the bug.
+            if (!$requestee->can_see_bug($bug_id)) {
+                ThrowUserError("flag_requestee_unauthorized",
+                               { flag_type => $flag_type,
+                                 requestee => $requestee,
+                                 bug_id    => $bug_id,
+                                 attach_id => $attach_id });
+            }
+            
+            # Throw an error if the target is a private attachment and
+            # the requestee isn't in the group of insiders who can see it.
+            if ($attach_id
+                && Param("insidergroup")
+                && $cgi->param('isprivate')
+                && !$requestee->in_group(Param("insidergroup")))
+            {
+                ThrowUserError("flag_requestee_unauthorized_attachment",
+                               { flag_type => $flag_type,
+                                 requestee => $requestee,
+                                 bug_id    => $bug_id,
+                                 attach_id => $attach_id });
             }
         }
 
@@ -536,9 +533,12 @@ sub sqlify_criteria {
         
         # Add inclusions to the query, which simply involves joining the table
         # by flag type ID and target product/component.
-        push(@$tables, "INNER JOIN flaginclusions AS i ON flagtypes.id = i.type_id");
-        push(@criteria, "(i.product_id = $product_id OR i.product_id IS NULL)");
-        push(@criteria, "(i.component_id = $component_id OR i.component_id IS NULL)");
+        push(@$tables, "INNER JOIN flaginclusions ON " .
+                       "flagtypes.id = flaginclusions.type_id");
+        push(@criteria, "(flaginclusions.product_id = $product_id " . 
+                        " OR flaginclusions.product_id IS NULL)");
+        push(@criteria, "(flaginclusions.component_id = $component_id " . 
+                        " OR flaginclusions.component_id IS NULL)");
         
         # Add exclusions to the query, which is more complicated.  First of all,
         # we do a LEFT JOIN so we don't miss flag types with no exclusions.
@@ -546,11 +546,13 @@ sub sqlify_criteria {
         # component.  However, since we want flag types that *aren't* on the
         # exclusions list, we add a WHERE criteria to use only records with
         # NULL exclusion type, i.e. without any exclusions.
-        my $join_clause = "flagtypes.id = e.type_id " .
-                          "AND (e.product_id = $product_id OR e.product_id IS NULL) " .
-                          "AND (e.component_id = $component_id OR e.component_id IS NULL)";
-        push(@$tables, "LEFT JOIN flagexclusions AS e ON ($join_clause)");
-        push(@criteria, "e.type_id IS NULL");
+        my $join_clause = "flagtypes.id = flagexclusions.type_id " . 
+                          "AND (flagexclusions.product_id = $product_id " . 
+                          "OR flagexclusions.product_id IS NULL) " . 
+                          "AND (flagexclusions.component_id = $component_id " .
+                          "OR flagexclusions.component_id IS NULL)";
+        push(@$tables, "LEFT JOIN flagexclusions ON ($join_clause)");
+        push(@criteria, "flagexclusions.type_id IS NULL");
     }
     if ($criteria->{group}) {
         my $gid = $criteria->{group};
