@@ -18,7 +18,6 @@
 # Rights Reserved.
 #
 # Contributor(s):    Myk Melez <myk@mozilla.org>
-#                    Frédéric Buclin <LpSolit@gmail.com>
 
 ################################################################################
 # Module Initialization
@@ -37,14 +36,9 @@ use Bugzilla::Util;
 
 use Date::Format;
 use Date::Parse;
-use File::Basename;
 
-use base qw(Exporter);
-
-@Bugzilla::Token::EXPORT = qw(issue_session_token check_token_data delete_token);
-
-# This module requires that its caller have said "require globals.pl" to import
-# relevant functions from that script.
+# This module requires that its caller have said "require CGI.pl" to import
+# relevant functions from that script and its companion globals.pl.
 
 ################################################################################
 # Constants
@@ -66,8 +60,8 @@ sub IssueEmailChangeToken {
 
     # Mail the user the token along with instructions for using it.
 
-    my $template = Bugzilla->template;
-    my $vars = {};
+    my $template = $::template;
+    my $vars = $::vars;
 
     $vars->{'oldemailaddress'} = $old_email . Param('emailsuffix');
     $vars->{'newemailaddress'} = $new_email . Param('emailsuffix');
@@ -109,7 +103,7 @@ sub IssuePasswordToken {
                     ON tokens.userid = profiles.userid
                     AND tokens.tokentype = 'password'
                     AND tokens.issuedate > NOW() - " .
-                    $dbh->sql_interval(10, 'MINUTE') . "
+                    $dbh->sql_interval('10 MINUTE') . "
                  WHERE " . $dbh->sql_istrcmp('login_name', $quotedloginname));
     my ($userid, $toosoon) = &::FetchSQLData();
 
@@ -121,8 +115,8 @@ sub IssuePasswordToken {
 
     # Mail the user the token along with instructions for using it.
     
-    my $template = Bugzilla->template;
-    my $vars = {};
+    my $template = $::template;
+    my $vars = $::vars;
 
     $vars->{'token'} = $token;
     $vars->{'emailaddress'} = $loginname . Param('emailsuffix');
@@ -138,7 +132,7 @@ sub IssuePasswordToken {
     Bugzilla::BugMail::MessageToMTA($message);
 }
 
-sub issue_session_token {
+sub IssueSessionToken {
     # Generates a random token, adds it to the tokens table, and returns
     # the token to the caller.
 
@@ -156,28 +150,24 @@ sub CleanTokenTable {
 }
 
 sub GenerateUniqueToken {
-    # Generates a unique random token.  Uses generate_random_password 
+    # Generates a unique random token.  Uses &GenerateRandomPassword 
     # for the tokens themselves and checks uniqueness by searching for
     # the token in the "tokens" table.  Gives up if it can't come up
     # with a token after about one hundred tries.
 
-    my ($table, $column) = @_;
-    
     my $token;
     my $duplicate = 1;
     my $tries = 0;
-    $table ||= "tokens";
-    $column ||= "token";
 
     my $dbh = Bugzilla->dbh;
-    my $sth = $dbh->prepare("SELECT userid FROM $table WHERE $column = ?");
+    my $sth = $dbh->prepare("SELECT userid FROM tokens WHERE token = ?");
 
     while ($duplicate) {
         ++$tries;
         if ($tries > 100) {
             ThrowCodeError("token_generation_error");
         }
-        $token = generate_random_password();
+        $token = &::GenerateRandomPassword();
         $sth->execute($token);
         $duplicate = $sth->fetchrow_array;
     }
@@ -190,10 +180,9 @@ sub Cancel {
     # This should only happen when the user accidentally makes a token request
     # or when a malicious hacker makes a token request on behalf of a user.
     
-    my ($token, $cancelaction, $vars) = @_;
+    my ($token, $cancelaction) = @_;
 
     my $dbh = Bugzilla->dbh;
-    $vars ||= {};
 
     # Quote the token for inclusion in SQL statements.
     my $quotedtoken = &::SqlQuote($token);
@@ -209,7 +198,8 @@ sub Cancel {
     # Get the email address of the Bugzilla maintainer.
     my $maintainer = Param('maintainer');
 
-    my $template = Bugzilla->template;
+    my $template = $::template;
+    my $vars = $::vars;
 
     $vars->{'emailaddress'} = $loginname . Param('emailsuffix');
     $vars->{'maintainer'} = $maintainer;
@@ -229,7 +219,7 @@ sub Cancel {
     Bugzilla::BugMail::MessageToMTA($message);
 
     # Delete the token from the database.
-    delete_token($token);
+    DeleteToken($token);
 }
 
 sub DeletePasswordTokens {
@@ -259,12 +249,11 @@ sub HasEmailChangeToken {
     return $token;
 }
 
-sub GetTokenData {
+sub GetTokenData($) {
     # Returns the userid, issuedate and eventdata for the specified token
 
     my ($token) = @_;
     return unless defined $token;
-    $token = clean_text($token);
     trick_taint($token);
     
     my $dbh = Bugzilla->dbh;
@@ -274,7 +263,7 @@ sub GetTokenData {
          WHERE  token = ?", undef, $token);
 }
 
-sub delete_token {
+sub DeleteToken($) {
     # Deletes specified token
 
     my ($token) = @_;
@@ -287,55 +276,11 @@ sub delete_token {
     $dbh->bz_unlock_tables();
 }
 
-# Given a token, makes sure it comes from the currently logged in user
-# and match the expected event. Returns 1 on success, else displays a warning.
-# Note: this routine must not be called while tables are locked as it will try
-# to lock some tables itself, see CleanTokenTable().
-sub check_token_data {
-    my ($token, $expected_action) = @_;
-    my $user = Bugzilla->user;
-    my $template = Bugzilla->template;
-    my $cgi = Bugzilla->cgi;
-
-    my ($creator_id, $date, $token_action) = GetTokenData($token);
-    unless ($creator_id
-            && $creator_id == $user->id
-            && $token_action eq $expected_action)
-    {
-        # Something is going wrong. Ask confirmation before processing.
-        # It is possible that someone tried to trick an administrator.
-        # In this case, we want to know his name!
-        require Bugzilla::User;
-
-        my $vars = {};
-        $vars->{'abuser'} = Bugzilla::User->new($creator_id)->identity;
-        $vars->{'token_action'} = $token_action;
-        $vars->{'expected_action'} = $expected_action;
-        $vars->{'script_name'} = basename($0);
-
-        # Now is a good time to remove old tokens from the DB.
-        CleanTokenTable();
-
-        # If no token was found, create a valid token for the given action.
-        unless ($creator_id) {
-            $token = issue_session_token($expected_action);
-            $cgi->param('token', $token);
-        }
-
-        print $cgi->header();
-
-        $template->process('admin/confirm-action.html.tmpl', $vars)
-          || ThrowTemplateError($template->error());
-        exit;
-    }
-    return 1;
-}
-
 ################################################################################
 # Internal Functions
 ################################################################################
 
-sub _create_token {
+sub _create_token($$$) {
     # Generates a unique token and inserts it into the database
     # Returns the token and the token timestamp
     my ($userid, $tokentype, $eventdata) = @_;
