@@ -67,10 +67,10 @@ use XML::Parser;
 use Data::Dumper;
 $Data::Dumper::Useqq = 1;
 use Bugzilla::BugMail;
+use Bugzilla::User;
 
 require "CGI.pl";
 require "globals.pl";
-$::lockcount = 0;
 
 GetVersionTable();
 
@@ -116,40 +116,6 @@ sub MailMessage {
 
   my $sendmessage = $header . $message . "\n";
   Bugzilla::BugMail::MessageToMTA($sendmessage);
-
-  Log($subject . " sent to: $to");
-}
-
-
-sub Log {
-    my ($str) = (@_);
-    Lock();
-    open(FID, ">>$datadir/maillog") || die "Can't write to $datadir/maillog";
-    print FID time2str("%D %H:%M", time()) . ": $str\n";
-    close FID;
-    Unlock();
-}
-
-sub Lock {
-    if ($::lockcount <= 0) {
-        $::lockcount = 0;
-        open(LOCKFID, ">>$datadir/maillock") || die "Can't open $datadir/maillock: $!";
-        my $val = flock(LOCKFID,2);
-        if (!$val) { # '2' is magic 'exclusive lock' const.
-            print Bugzilla->cgi->header();
-            print "Lock failed: $val\n";
-        }
-        chmod 0666, "$datadir/maillock";
-    }
-    $::lockcount++;
-}
-
-sub Unlock {
-    $::lockcount--;
-    if ($::lockcount <= 0) {
-        flock(LOCKFID,8);       # '8' is magic 'unlock' const.
-        close LOCKFID;
-    }
 }
 
 
@@ -162,6 +128,7 @@ $xml =~ s/^.+(<\?xml version.+)$/$1/s;
 
 my $parser = new XML::Parser(Style => 'Tree');
 my $tree = $parser->parse($xml);
+my $dbh = Bugzilla->dbh;
 
 my $maintainer;
 if (defined $tree->[1][0]->{'maintainer'}) {
@@ -201,7 +168,7 @@ unless ( Param("move-enabled") ) {
   exit;
 }
 
-my $exporterid = DBname_to_id($exporter);
+my $exporterid = login_to_id($exporter);
 if ( ! $exporterid ) {
   my $subject = "Bug import error: invalid exporter";
   my $message = "The user <$tree->[1][0]->{'exporter'}> who tried to move\n";
@@ -339,13 +306,21 @@ for (my $k=1 ; $k <= $bugqty ; $k++) {
 
   my @query = ();
   my @values = ();
-  foreach my $field ( qw(creation_ts delta_ts status_whiteboard) ) {
+  foreach my $field ( qw(creation_ts status_whiteboard) ) {
       if ( (defined $bug_fields{$field}) && ($bug_fields{$field}) ){
         push (@query, "$field");
         push (@values, SqlQuote($bug_fields{$field}));
       }
   }
 
+  push (@query, "delta_ts");
+  if ( (defined $bug_fields{'delta_ts'}) && ($bug_fields{'delta_ts'}) ){
+      push (@values, SqlQuote($bug_fields{'delta_ts'}));
+  }
+  else {
+      push (@values, "NOW()");
+  }
+
   if ( (defined $bug_fields{'bug_file_loc'}) && ($bug_fields{'bug_file_loc'}) ){
       push (@query, "bug_file_loc");
       push (@values, SqlQuote($bug_fields{'bug_file_loc'}));
@@ -412,8 +387,8 @@ for (my $k=1 ; $k <= $bugqty ; $k++) {
   }
 
   if (defined  ($::versions{$product[0]} ) &&
-     (my @version = grep /^$bug_fields{'version'}$/i, 
-                         @{$::versions{$product[0]}}) ){
+     (my @version = grep(lc($_) eq lc($bug_fields{'version'}), 
+                         @{$::versions{$product[0]}})) ){
     push (@values, SqlQuote($version[0]) );
     push (@query, "version");
   } else {
@@ -424,7 +399,7 @@ for (my $k=1 ; $k <= $bugqty ; $k++) {
   }
 
   if (defined ($bug_fields{'priority'}) &&
-       (my @priority = grep /^$bug_fields{'priority'}$/i, @::legal_priority) ){
+       (my @priority = grep(lc($_) eq lc($bug_fields{'priority'}), @::legal_priority)) ){
     push (@values, SqlQuote($priority[0]) );
     push (@query, "priority");
   } else {
@@ -436,7 +411,7 @@ for (my $k=1 ; $k <= $bugqty ; $k++) {
   }
 
   if (defined ($bug_fields{'rep_platform'}) &&
-       (my @platform = grep /^$bug_fields{'rep_platform'}$/i, @::legal_platform) ){
+       (my @platform = grep(lc($_) eq lc($bug_fields{'rep_platform'}), @::legal_platform)) ){
     push (@values, SqlQuote($platform[0]) );
     push (@query, "rep_platform");
   } else {
@@ -449,7 +424,7 @@ for (my $k=1 ; $k <= $bugqty ; $k++) {
   }
 
   if (defined ($bug_fields{'op_sys'}) &&
-     (my @opsys = grep /^$bug_fields{'op_sys'}$/i, @::legal_opsys) ){
+     (my @opsys = grep(lc($_) eq lc($bug_fields{'op_sys'}), @::legal_opsys)) ){
     push (@values, SqlQuote($opsys[0]) );
     push (@query, "op_sys");
   } else {
@@ -462,8 +437,8 @@ for (my $k=1 ; $k <= $bugqty ; $k++) {
 
   if (Param("usetargetmilestone")) {
     if (defined  ($::target_milestone{$product[0]} ) &&
-       (my @tm = grep /^$bug_fields{'target_milestone'}$/i, 
-                       @{$::target_milestone{$product[0]}}) ){
+       (my @tm = grep(lc($_) eq lc($bug_fields{'target_milestone'}), 
+                       @{$::target_milestone{$product[0]}})) ){
       push (@values, SqlQuote($tm[0]) );
       push (@query, "target_milestone");
     } else {
@@ -482,8 +457,8 @@ for (my $k=1 ; $k <= $bugqty ; $k++) {
   }
 
   if (defined ($bug_fields{'bug_severity'}) &&
-       (my @severity= grep /^$bug_fields{'bug_severity'}$/i, 
-                           @::legal_severity) ){
+       (my @severity= grep(lc($_) eq lc($bug_fields{'bug_severity'}), 
+                           @::legal_severity)) ){
     push (@values, SqlQuote($severity[0]) );
     push (@query, "bug_severity");
   } else {
@@ -495,7 +470,7 @@ for (my $k=1 ; $k <= $bugqty ; $k++) {
     $err .= ". Setting to default severity \"normal\".\n";
   }
 
-  my $reporterid = DBname_to_id($bug_fields{'reporter'});
+  my $reporterid = login_to_id($bug_fields{'reporter'});
   if ( ($bug_fields{'reporter'}) && ( $reporterid ) ) {
     push (@values, SqlQuote($reporterid));
     push (@query, "reporter");
@@ -514,39 +489,39 @@ for (my $k=1 ; $k <= $bugqty ; $k++) {
 
   my $changed_owner = 0;
   if ( ($bug_fields{'assigned_to'}) && 
-       ( DBname_to_id($bug_fields{'assigned_to'})) ) {
-    push (@values, SqlQuote(DBname_to_id($bug_fields{'assigned_to'})));
+       ( login_to_id($bug_fields{'assigned_to'})) ) {
+    push (@values, SqlQuote(login_to_id($bug_fields{'assigned_to'})));
     push (@query, "assigned_to");
   } else {
     push (@values, SqlQuote($exporterid) );
     push (@query, "assigned_to");
     $changed_owner = 1;
-    $err .= "The original owner of this bug does not have\n";
+    $err .= "The original assignee of this bug does not have\n";
     $err .= "   an account here. Reassigning to the person who moved\n";
     $err .= "   it here, $exporter.\n";
     if ( $bug_fields{'assigned_to'} ) {
-      $err .= "   Previous owner was $bug_fields{'assigned_to'}.\n";
+      $err .= "   Previous assignee was $bug_fields{'assigned_to'}.\n";
     } else {
-      $err .= "   Previous owner is unknown.\n";
+      $err .= "   Previous assignee is unknown.\n";
     }
   }
 
   my @resolution;
   if (defined ($bug_fields{'resolution'}) &&
-       (@resolution= grep /^$bug_fields{'resolution'}$/i, @::legal_resolution) ){
+       (@resolution= grep(lc($_) eq lc($bug_fields{'resolution'}), @::legal_resolution)) ){
     push (@values, SqlQuote($resolution[0]) );
     push (@query, "resolution");
   } elsif ( (defined $bug_fields{'resolution'}) && (!$resolution[0]) ){
     $err .= "Unknown resolution \"$bug_fields{'resolution'}\".\n";
   }
 
-  # if the bug's owner changed, mark the bug NEW, unless a valid 
+  # if the bug's assignee changed, mark the bug NEW, unless a valid 
   # resolution is set, which indicates that the bug should be closed.
   #
   if ( ($changed_owner) && (!$resolution[0]) ) {
     push (@values, SqlQuote("NEW"));
     push (@query, "bug_status");
-    $err .= "Bug assigned to new owner, setting status to \"NEW\".\n";
+    $err .= "Bug reassigned, setting status to \"NEW\".\n";
     $err .= "   Previous status was \"";
     $err .= (defined $bug_fields{'bug_status'})?
                      $bug_fields{'bug_status'}:"unknown";
@@ -561,7 +536,7 @@ for (my $k=1 ; $k <= $bugqty ; $k++) {
                      $bug_fields{'bug_status'}:"unknown";
     $err .= "\".\n";
   } elsif (defined ($bug_fields{'bug_status'}) &&
-       (my @status = grep /^$bug_fields{'bug_status'}$/i, @::legal_bug_status) ){
+       (my @status = grep(lc($_) eq lc($bug_fields{'bug_status'}), @::legal_bug_status)) ){
     #if a bug status was set then use it, if its legal
     push (@values, SqlQuote($status[0]));
     push (@query, "bug_status");
@@ -578,7 +553,7 @@ for (my $k=1 ; $k <= $bugqty ; $k++) {
   if (Param("useqacontact")) {
     my $qa_contact;
     if ( (defined $bug_fields{'qa_contact'}) &&
-         ($qa_contact  = DBname_to_id($bug_fields{'qa_contact'})) ){
+         ($qa_contact  = login_to_id($bug_fields{'qa_contact'})) ){
       push (@values, $qa_contact);
       push (@query, "qa_contact");
     } else {
@@ -601,13 +576,12 @@ for (my $k=1 ; $k <= $bugqty ; $k++) {
                . join (",\n", @values)
                . "\n)\n";
   SendSQL($query);
-  SendSQL("select LAST_INSERT_ID()");
-  my $id = FetchOneColumn();
+  my $id = $dbh->bz_last_key('bugs', 'bug_id');
 
   if (defined $bug_fields{'cc'}) {
     foreach my $person (split(/[ ,]/, $bug_fields{'cc'})) {
       my $uid;
-      if ( ($person ne "") && ($uid = DBname_to_id($person)) ) {
+      if ( ($person ne "") && ($uid = login_to_id($person)) ) {
         SendSQL("insert into cc (bug_id, who) values ($id, " . SqlQuote($uid) .")");
       }
     }
