@@ -34,7 +34,6 @@ use Bugzilla::Constants;
 use Bugzilla::Search;
 use Bugzilla::User;
 use Bugzilla::BugMail;
-use Bugzilla::Util;
 
 # create some handles that we'll need
 my $template = Bugzilla->template;
@@ -51,7 +50,7 @@ my @seen_schedules = ();
 # allow the database to keep their SQL compiled.
 my $sth_run_queries =
     $dbh->prepare("SELECT " .
-                  "query_name, title, onemailperbug " .
+                  "id, query_name, title, onemailperbug " .
                   "FROM whine_queries " .
                   "WHERE eventid=? " .
                   "ORDER BY sortkey");
@@ -94,6 +93,12 @@ my $sth_schedules_by_event = $dbh->prepare(
 # After that, it looks over each user to see if they have schedules that need
 # running, then runs those and generates the email messages.
 
+# exit quietly if the system is shut down
+if (Param('shutdownhtml')) {
+    exit;
+}
+
+
 # Send whines from the address in the 'maintainer' Parameter so that all
 # Bugzilla-originated mail appears to come from a single address.
 my $fromaddress = Param('maintainer');
@@ -165,21 +170,20 @@ while (my ($schedule_id, $day, $time) = $sched_h->fetchrow_array) {
         # A time greater than now means it still has to run today
         elsif ($time >= $now_hour) {
             # set it to today + number of hours
-            $sth = $dbh->prepare("UPDATE whine_schedules " .
-                                 "SET run_next = CURRENT_DATE + " .
-                                 $dbh->sql_interval('?', 'HOUR') .
-                                 " WHERE id = ?");
+            $sth = $dbh->prepare( "UPDATE whine_schedules " .
+                   "SET run_next = CURRENT_DATE() + " .
+                   $dbh->sql_interval('? HOUR') .
+                   " WHERE id = ?");
             $sth->execute($time, $schedule_id);
         }
         # the target time is less than the current time
         else { # set it for the next applicable day
-            $day = &get_next_date($day);
-            $sth = $dbh->prepare("UPDATE whine_schedules " .
-                                 "SET run_next = (CURRENT_DATE + " .
-                                 $dbh->sql_interval('?', 'DAY') . ") + " .
-                                 $dbh->sql_interval('?', 'HOUR') .
-                                 " WHERE id = ?");
-            $sth->execute($day, $time, $schedule_id);
+            my $nextdate = &get_next_date($day);
+            $sth = $dbh->prepare( "UPDATE whine_schedules " .
+                   "SET run_next = ? + " .
+                   $dbh->sql_interval('? HOUR') .
+                   " WHERE id = ?");
+            $sth->execute($nextdate, $time, $schedule_id);
         }
 
     }
@@ -191,11 +195,10 @@ while (my ($schedule_id, $day, $time) = $sched_h->fetchrow_array) {
         # midnight
         my $target_time = ($time =~ /^\d+$/) ? $time : 0;
 
-        $sth = $dbh->prepare("UPDATE whine_schedules " .
-                             "SET run_next = (CURRENT_DATE + " .
-                             $dbh->sql_interval('?', 'DAY') . ") + " .
-                             $dbh->sql_interval('?', 'HOUR') .
-                             " WHERE id = ?");
+        $sth = $dbh->prepare( "UPDATE whine_schedules " .
+               "SET run_next = ? + " .
+               $dbh->sql_interval('? HOUR') .
+               " WHERE id = ?");
         $sth->execute($target_date, $target_time, $schedule_id);
     }
 }
@@ -225,10 +228,10 @@ sub get_next_event {
 
         $dbh->bz_lock_tables('whine_schedules WRITE',
                              'whine_events READ',
-                             'profiles READ',
+                             'profiles WRITE',
                              'groups READ',
                              'group_group_map READ',
-                             'user_group_map READ');
+                             'user_group_map WRITE');
 
         # Get the event ID for the first pending schedule
         $sth_next_scheduled_event->execute;
@@ -237,7 +240,8 @@ sub get_next_event {
         return undef unless $fetched;
         my ($eventid, $owner_id, $subject, $body) = @{$fetched};
 
-        my $owner = Bugzilla::User->new($owner_id);
+        my $owner = Bugzilla::User->new($owner_id,
+                                        DERIVE_GROUPS_TABLES_ALREADY_LOCKED);
 
         my $whineatothers = $owner->in_group('bz_canusewhineatothers');
 
@@ -259,7 +263,7 @@ sub get_next_event {
                             $user_objects{$mailto} = $owner;
                         }
                         elsif ($whineatothers) {
-                            $user_objects{$mailto} = Bugzilla::User->new($mailto);
+                            $user_objects{$mailto} = Bugzilla::User->new($mailto,DERIVE_GROUPS_TABLES_ALREADY_LOCKED);
                         }
                     }
                 }
@@ -271,17 +275,14 @@ sub get_next_event {
                     my $group_id = Bugzilla::Group::ValidateGroupName(
                         $groupname, $owner);
                     if ($group_id) {
-                        my $glist = join(',',
-                            @{Bugzilla::User->flatten_group_membership(
-                            $group_id)});
                         $sth = $dbh->prepare("SELECT user_id FROM " .
                                              "user_group_map " .
-                                             "WHERE group_id IN ($glist)");
-                        $sth->execute();
+                                             "WHERE group_id=?");
+                        $sth->execute($group_id);
                         for my $row (@{$sth->fetchall_arrayref}) {
                             if (not defined $user_objects{$row->[0]}) {
                                 $user_objects{$row->[0]} =
-                                    Bugzilla::User->new($row->[0]);
+                                    Bugzilla::User->new($row->[0],DERIVE_GROUPS_TABLES_ALREADY_LOCKED);
                             }
                         }
                     }
@@ -323,7 +324,7 @@ while (my $event = get_next_event) {
 
     # We loop for each target user because some of the queries will be using
     # subjective pronouns
-    $dbh = Bugzilla->switch_to_shadow_db();
+    Bugzilla->switch_to_shadow_db();
     for my $target (@{$event->{'mailto'}}) {
         my $args = {
             'subject'     => $event->{'subject'},
@@ -348,7 +349,7 @@ while (my $event = get_next_event) {
 
         mail($args);
     }
-    $dbh = Bugzilla->switch_to_main_db();
+    Bugzilla->switch_to_main_db();
 }
 
 ################################################################################
@@ -404,7 +405,7 @@ sub mail {
 
     # now produce a ready-to-mail mime-encoded message
 
-    $args->{'boundary'} = "----------" . $$ . "--" . time() . "-----";
+    $args->{'boundary'} = "-----=====-----" . $$ . "--" . time() . "-----";
 
     $template->process("whine/multipart-mime.txt.tmpl", $args, \$msg)
         or die($template->error());
@@ -425,19 +426,18 @@ sub run_queries {
     my $return_queries = [];
 
     $sth_run_queries->execute($args->{'eventid'});
-    my @queries = ();
+    my $queries = {};
     for (@{$sth_run_queries->fetchall_arrayref}) {
-        push(@queries,
-            {
-              'name'          => $_->[0],
-              'title'         => $_->[1],
-              'onemailperbug' => $_->[2],
-              'bugs'          => [],
-            }
-        );
+        $queries->{$_->[0]} = {
+            'name'          => $_->[1],
+            'title'         => $_->[2],
+            'onemailperbug' => $_->[3],
+            'bugs'          => [],
+        };
     }
 
-    foreach my $thisquery (@queries) {
+    for my $query_id (keys %{$queries}) {
+        my $thisquery = $queries->{$query_id};
         next unless $thisquery->{'name'};   # named query is blank
 
         my $savedquery = get_query($thisquery->{'name'}, $args->{'author'});
@@ -565,10 +565,6 @@ sub reset_timer {
     $sth->execute($schedule_id);
     my ($run_day, $run_time) = $sth->fetchrow_array;
 
-    # It may happen that the run_time field is NULL or blank due to
-    # a bug in editwhines.cgi when this field was initially 0.
-    $run_time ||= 0;
-
     my $run_today = 0;
     my $minute_offset = 0;
 
@@ -600,25 +596,23 @@ sub reset_timer {
 
         my $nextdate = &get_next_date($run_day);
 
-        $sth = $dbh->prepare("UPDATE whine_schedules " .
-                             "SET run_next = (CURRENT_DATE + " .
-                             $dbh->sql_interval('?', 'DAY') . ") + " .
-                             $dbh->sql_interval('?', 'HOUR') .
-                             " WHERE id = ?");
+        $sth = $dbh->prepare( "UPDATE whine_schedules " .
+                              "SET run_next = ? + " .
+                              $dbh->sql_interval('? HOUR') .
+                              " WHERE id = ?");
         $sth->execute($nextdate, $target_time, $schedule_id);
         return;
     }
 
+    # Scheduling is done in terms of whole minutes, so we use subtraction to
+    # drop the seconds from the time.
     if ($minute_offset > 0) {
-        # Scheduling is done in terms of whole minutes.
-        my $next_run = $dbh->selectrow_array('SELECT NOW() + ' .
-                                             $dbh->sql_interval('?', 'MINUTE'),
-                                             undef, $minute_offset);
-        $next_run = format_time($next_run, "%Y-%m-%d %R");
-
         $sth = $dbh->prepare("UPDATE whine_schedules " .
-                             "SET run_next = ? WHERE id = ?");
-        $sth->execute($next_run, $schedule_id);
+                             "SET run_next = NOW() + " .
+                             $dbh->sql_interval('? MINUTE') . " - " .
+                             $dbh->sql_interval('SECOND(NOW()) SECOND') .
+                             " WHERE id = ?");
+        $sth->execute($minute_offset, $schedule_id);
     } else {
         # The minute offset is zero or less, which is not supposed to happen.
         # complain to STDERR
@@ -642,7 +636,7 @@ sub null_schedule {
 # time a schedule should run, excluding today
 #
 # It takes a run_day argument (see check_today, above, for an explanation),
-# and returns an integer, representing a number of days.
+# and returns an SQL date
 sub get_next_date {
     my $day = shift;
 
@@ -695,5 +689,11 @@ sub get_next_date {
             $add_days += $daysinmonth[$now_month];
         }
     }
-    return $add_days;
+
+    # Get a date in whatever format the database will accept
+    $sth = $dbh->prepare("SELECT CURRENT_DATE() + " .
+                         $dbh->sql_interval('? DAY'));
+    $sth->execute($add_days);
+    return $sth->fetch->[0];
 }
+
